# for encoding categorical variables
from feature_engine.encoding import OneHotEncoder, RareLabelEncoder

# for imputation
from feature_engine.imputation import AddMissingIndicator, CategoricalImputer, MeanMedianImputer

# to build the models
from sklearn.linear_model import LogisticRegression
from sklearn.pipeline import Pipeline

# feature scaling
from sklearn.preprocessing import StandardScaler

from model.config.core import config
from model.processing.features import ExtractLetterTransformer

titanic_pipe = Pipeline([
    # ===== IMPUTATION =====
    # impute categorical variables with string missing
    ('categorical_imputation', CategoricalImputer(
        imputation_method='missing',
        variables=config.model_config.categorical_vars)),

    # add missing indicator to numerical variables
    ('missing_indicator', AddMissingIndicator(
        variables=config.model_config.numerical_vars)),

    # impute numerical variables with the median
    ('median_imputation', MeanMedianImputer(
        imputation_method='median',
        variables=config.model_config.numerical_vars)),

    # Extract letter from cabin
    ('extract_letter', ExtractLetterTransformer(
        variables=config.model_config.vars_to_extract_letter)),

    # == CATEGORICAL ENCODING ======
    # remove categories present in less than 5% of the observations (0.05)
    # group them in one category called 'Rare'
    ('rare_label_encoder', RareLabelEncoder(
        tol=0.05,
        n_categories=1,
        variables=config.model_config.categorical_vars)),

    # encode categorical variables using one hot encoding into k-1 variables
    ('categorical_encoder', OneHotEncoder(
        drop_last=True,
        variables=config.model_config.categorical_vars)),

    # scale
    ('scaler', StandardScaler()),

    ('Logit', LogisticRegression(C=0.0005, random_state=0)),
])



from model.config.core import config
from model.processing.features import ExtractLetterTransformer


def test_temporal_variable_transformer(sample_input_data):
    # Given
    transformer = ExtractLetterTransformer(
        variables=config.model_config.vars_to_extract_letter,  # cabin
    )
    assert sample_input_data["cabin"].iat[3] == "E33"

    # When
    subject = transformer.fit_transform(sample_input_data)

    # Then
    assert subject["cabin"].iat[3] == "E"



from sklearn.base import BaseEstimator, TransformerMixin


class ExtractLetterTransformer(BaseEstimator, TransformerMixin):
    # Extract fist letter of variable

    def __init__(self, variables):

        if not isinstance(variables, list):
            raise ValueError('variables should be a list')

        self.variables = variables

    def fit(self, X, y=None):
        # we need this step to fit the sklearn pipeline
        return self

    def transform(self, X):

        # so that we do not over-write the original dataframe
        X = X.copy()

        for feature in self.variables:
            X[feature] = X[feature].str[0]

        return X



import numpy as np

from sklearn.metrics import accuracy_score
from model.predict import make_prediction


def test_prediction_against_truth_value(sample_input_data):
    # When
    estimated_results = make_prediction(input_data=sample_input_data)

    # Then
    predictions = estimated_results.get("predictions")
    assert isinstance(predictions, np.ndarray)
    assert isinstance(predictions[0], np.int64)
    assert estimated_results.get("errors") is None
    _predictions = list(predictions)
    y_true = sample_input_data["survived"]
    accuracy = accuracy_score(y_true, _predictions)
    assert accuracy > 0.7
